#include <Evas.h>
#include <Ecore.h>
#include <Ecore_Evas.h>
#include <Emotion.h>

char *file = "dvd:/";
Ecore_Evas *ecore_evas;
Evas_Object *background;
Evas_Object *video;

/* if the window manager requests a delete - quit cleanly */
static void
canvas_delete_request(Ecore_Evas *ecore_evas_in)
{
   ecore_main_loop_quit();
}

/* if the canvas is resized - resize the video too */
static void
canvas_resize(Ecore_Evas *ecore_evas_in)
{
   Evas_Coord x, y, w, h;
   int vw, vh;
   double ratio;
   
   /* get the current canvas size */
   evas_output_viewport_get(ecore_evas_get(ecore_evas), NULL, NULL, &w, &h);

   /* resize the black background rectangle */
   evas_object_move(background, 0, 0);
   evas_object_resize(background, w, h);

   /* resize the video object AND retain aspect ratio */
   /* get the video size in pixels */
   emotion_object_size_get(video, &vw, &vh);
   /* get the video ratio */
   ratio = emotion_object_ratio_get(video);
   /* if the ratio > 0.0 then the video wants awidth / height ratio on the */
   /* screen as returned indicated by this ratio value. this is irrespective */
   /* of the pixel size of the video and indicates the video wants to scale */
   if (ratio > 0.0)
     {
	x = 0;
	y = (h - (w / ratio)) / 2;
	if (y < 0)
	  {
	     y = 0;
	     x = (w - (h * ratio)) / 2;
	     w = h * ratio;
	  }
	else
	  h = w / ratio;
	evas_object_move(video, x, y);
	evas_object_resize(video, w, h);
     }
   else
     {
	if (vh > 0)
	  /* generate ratio from the pixel size */
	  ratio = (double)vw / (double)vh;
	else
	  ratio = 1.0;
	x = 0;
	y = (h - (w / ratio)) / 2;
	if (y < 0)
	  {
	     y = 0;
	     x = (w - (h * ratio)) / 2;
	     w = h * ratio;
	  }
	else
	  h = w / ratio;
	evas_object_move(video, 0, 0);
	evas_object_resize(video, w, h);
     }
}

/* this function is set to be called whenever a key press event happens */
/* on the object that has this callback set */
void key_press(void *data, Evas *canvas_in, Evas_Object *obj, void *event_info)
{
   /* the event type given to this callback is the Evas_Event_Key_Down */
   Evas_Event_Key_Down *ev;
   
   ev = (Evas_Event_Key_Down *)event_info;
   /* if we hit "Escape" or "q" lets quit */
   if ((!strcmp(ev->keyname, "Escape")) || 
       (!strcmp(ev->keyname, "q")))
     ecore_main_loop_quit();
   /* if we his "comma" (also the "<"  key on qwerty keyboards) rewind by */
   /* 10 seconds */
   else if (!strcmp(ev->keyname, "comma"))
     {
	double pos;
	
	pos = emotion_object_position_get(video);
	emotion_object_position_set(video, pos - 10.0);
     }
   /* if we his "period" (also the ">"  key on qwerty keyboards) forward by */
   /* 10 seconds */
   else if (!strcmp(ev->keyname, "period"))
     {
	double pos;
	
	pos = emotion_object_position_get(video);
	emotion_object_position_set(video, pos + 10.0);
     }
   /* allow our app to toggle "fullscreen" mode if you hit "f" */
   else if (!strcmp(ev->keyname, "f"))
     {
	if (!ecore_evas_fullscreen_get(ecore_evas))
	  ecore_evas_fullscreen_set(ecore_evas, 1);
	else
	  ecore_evas_fullscreen_set(ecore_evas, 0);
     }
}
    
/* the main function of the program */
int main(int argc, char **argv)
{
   int i;
   int opt_fullscreen = 0;
   
   /* properly parse command-line arguments */
   for (i = 1; i < argc; i++)
     {
	/* start in fullscreen mode */
	if (!strcmp(argv[i], "-f"))
	  opt_fullscreen = 1;
	else
	  file = argv[i];
     }
   /* create a canvas, display it, set a title, callbacks to call on resize */
   /* or if the window manager asks it to be deleted */
   ecore_evas_init();
   ecore_evas = ecore_evas_software_x11_new(NULL, 0, 0, 0, 800, 600);
   ecore_evas_callback_delete_request_set(ecore_evas, canvas_delete_request);
   ecore_evas_callback_resize_set(ecore_evas, canvas_resize);   
   ecore_evas_title_set(ecore_evas, "My DVD Player");
   ecore_evas_name_class_set(ecore_evas, "my_dvd_player", "My_DVD_Player");
   /* if our start option said to run fullscreen, we should */
   if (opt_fullscreen) ecore_evas_fullscreen_set(ecore_evas, 1);
   ecore_evas_show(ecore_evas);

   /* create a black background object */
   background = evas_object_rectangle_add(ecore_evas_get(ecore_evas));
   evas_object_color_set(background, 0, 0, 0, 255);
   evas_object_show(background);
   /* focuses the background so key events go to this object and call the */
   /* key_press function every time a key goes down */
   evas_object_focus_set(background, 1);
   evas_object_event_callback_add(background, EVAS_CALLBACK_KEY_DOWN, key_press, NULL);
   
   /* create a video object */
   video = emotion_object_add(ecore_evas_get(ecore_evas));
   emotion_object_file_set(video, file);
   emotion_object_play_set(video, 1);
   evas_object_show(video);

   /* force an initial resize */
   canvas_resize(ecore_evas);
   
   /* run the main loop of the program - playing, drawing, handling events */
   ecore_main_loop_begin();
   
   /* if we exit the main loop we will shut down */
   ecore_evas_shutdown();
}
